from orca_rs.outputs.casscf import CASSCFOutput
from orca_rs.outputs.tddft import TDDFTOutput

def parse_tddft(filename: str) -> TDDFTOutput:
    """Parse orca output file"""


def parse_casscf(filename: str) -> CASSCFOutput:
    """Parse orca output file"""
